/*
   From the ReceiveDemo example of IRremote library
   This is the original info and copywrite.

   Demonstrates receiving IR codes with the IRremote library.
   If debug button is pressed (pin connected to ground) a long output is generated.

    This file is part of Arduino-IRremote https://github.com/Arduino-IRremote/Arduino-IRremote.

 ************************************************************************************
   MIT License

   Copyright (c) 2020-2022 Armin Joachimsmeyer

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is furnished
   to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
   INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
   PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
   HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
   OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

 ************************************************************************************
*/

/*
 * Specific for XIAO BLE and XIAO BLE Sense boards from davekw7x
 * The idea is to give a reasonable test sketch without having  to
 * wade  through the conditional #if stuff in the sketch or in
 * the project's "PinDefinitionsAndMore" header.
 * 
 * After you get this to work, then look at all of the examples
 * and pick the one(s) you need to get started on your "real"
 * application.
 * 
 * Tested with my XIAO BLE and IRremote library version 3.6.1
 * 
 * davekw7x,
 * May, 2022
 */



#define INFO // To see valuable informations from universal decoder for pulse width or pulse distance protocols

// If you define _IR_MEASURE_TIMING, you will see pulses
// on the Test pin.  Period = 50 us (with some unavoidable jitter)
#define _IR_MEASURE_TIMING
#define _IR_TIMING_TEST_PIN 7

// If you hold this pin low, you will see detailed pulse information
// for valid codes as well as for invalid  codes
#define DEBUG_BUTTON_PIN    5

// This is the input from the IR Receiver module
#define IR_RECEIVE_PIN      2

// This allows built-in LED to flicker when bit streams are being received
#define ENABLE_LED_FEEDBACK true

// You will see a short burst on this pin when bit streams are being received
#define TONE_PIN 4

#include <IRremote.h>


void setup() {
  pinMode(DEBUG_BUTTON_PIN, INPUT_PULLUP);

  Serial.begin(115200);
  while (!Serial);
  Serial.println("\r\nIR Receive Demo compiled on " __DATE__ " at " __TIME__);
  Serial.println("Using IRremote library version " VERSION_IRREMOTE);
#ifdef ARDUINO_ARCH_MBED
  Serial.println("ARDUINO_ARCH_MBED is defined");
#else
  Serial.println("ARDUINO_ARCH_MBED is not defined.  Are you sure you selected the right board?");
#endif
  Serial.print("RAW_BUFFER_LENGTH  = "); Serial.println(RAW_BUFFER_LENGTH);

  Serial.print("IR_RECEIVE_PIN      = "); Serial.println(IR_RECEIVE_PIN);
  Serial.print("ENABLE_LED_FEEDBACK = "); Serial.println(ENABLE_LED_FEEDBACK);
  Serial.print("LED_BUILTIN         = "); Serial.println(LED_BUILTIN);
  Serial.print("TONE_PIN            = "); Serial.println(TONE_PIN);
  Serial.print("DEBUG_BUTTON_PIN    = "); Serial.println(DEBUG_BUTTON_PIN);
#ifdef _IR_MEASURE_TIMING
  Serial.println("_IR_MEASURE_TIMING is defined.");
  Serial.print("_IR_TIMING_TEST_PIN = "); Serial.println(_IR_TIMING_TEST_PIN);
  pinMode(_IR_TIMING_TEST_PIN, OUTPUT);
#else
  Serial.println("_IR_MEASURE_TIMING is not defined.");
#endif

  // In case the timer interrupt service routine crashes on setup, give a clue
  // to the user what's going on.
  Serial.println("Enabling IRin...");

  // Start the receiver and if not 3rd parameter specified, take LED_BUILTIN pin from the internal boards definition as default feedback LED
  IrReceiver.begin(IR_RECEIVE_PIN, ENABLE_LED_FEEDBACK);

  Serial.println("Ready to receive IR signals of protocols:");
  printActiveIRProtocols(&Serial);
  Serial.print("at pin ");
  Serial.println(IR_RECEIVE_PIN);

  // infos for receive
  Serial.print(RECORD_GAP_MICROS);
  Serial.println(" us is the (minimum) gap, after which the start of a new IR packet is assumed");
  Serial.print(MARK_EXCESS_MICROS);
  Serial.println(" us are subtracted from all marks and added to all spaces for decoding");
  //pinMode(TONE_PIN, OUTPUT);
}
void loop() {
  /*
     Check if received data is available and if yes, try to decode it.
     Decoded result is in the IrReceiver.decodedIRData structure.

     E.g. command is in IrReceiver.decodedIRData.command
     address is in command is in IrReceiver.decodedIRData.address
     and up to 32 bit raw data in IrReceiver.decodedIRData.decodedRawData
  */
  if (IrReceiver.decode()) {
    Serial.println();

    if (IrReceiver.decodedIRData.flags & IRDATA_FLAGS_WAS_OVERFLOW) {
      Serial.println("Overflow detected");
      Serial.print("Try to increase the \"RAW_BUFFER_LENGTH\" value of ");
      Serial.print(RAW_BUFFER_LENGTH); Serial.println(" in file " __FILE__);
      // see also https://github.com/Arduino-IRremote/Arduino-IRremote#modifying-compile-options-with-sloeber-ide
      /*
         do double beep
      */
      IrReceiver.stop(); // ESP32 uses another timer for tone()
      tone(TONE_PIN, 1100, 10);
      delay(50);
      tone(TONE_PIN, 1100, 10);
      delay(50);
      IrReceiver.start(100000); // to compensate for 100 ms stop of receiver. This enables a correct gap measurement.
    }
    else {
      // No overflow: Print a short summary of received data
      Serial.print(millis()); Serial.print(": ");
      IrReceiver.printIRResultShort(&Serial);

      if (IrReceiver.decodedIRData.protocol == UNKNOWN || digitalRead(DEBUG_BUTTON_PIN) == LOW) {
        // We have an unknown protocol, or the debug pin is low, print more info
        IrReceiver.printIRResultRawFormatted(&Serial, true);
      }
    }
    if (IrReceiver.decodedIRData.protocol != UNKNOWN) {
      /*
         If a valid protocol was received, play tone, wait and restore IR timer.
         Otherwise do not play a tone to get exact gap time between transmissions.
         This will give the next CheckForRecordGapsMicros() call a chance to eventually propose a change of the current RECORD_GAP_MICROS value.
      */
      IrReceiver.stop(); // ESP32 uses another timer for tone()
      tone(TONE_PIN, 2200, 8);
      delay(8);
      IrReceiver.start(8000); // to compensate for 8 ms stop of receiver. This enables a correct gap measurement.
    } // protocol != UNKNOWN

    /*
       !!!Important!!! Enable receiving of the next value,
       since receiving has stopped after the end of the current received data packet.
    */
    IrReceiver.resume();

    /*
       Finally check the received data and perform actions according to the received address and commands
       For example:
    */
    if (IrReceiver.decodedIRData.address == 0x707) {
      switch (IrReceiver.decodedIRData.command) {
        case 0x07:
          Serial.println("Samsung: Increase Volume");
          break;
        
        case 0x0B:
          Serial.println("Samsung: Decrease Volume");
          break;
        
        case 0x68:
          Serial.println("Samsung: OK");
          break;

        // Cases for other keys' action for other keys here
          
        default:
          break;
      } // End switch()
    } // address == 0x707
  } // if (IrReceiver.decode())
}
